@@ -6,10 +6,10 @@ import argparse
 import datetime
 
 from ..expdb import expdb
-from ..tools import badwulf_attribution
-from ..tools import print_datasets
-from ..tools import format_bytes
+from ..expdb import format_datasets
+from ..expdb import format_size
 from ..tools import findport
+from ..tools import badwulf_attribution
 
 class dbmanager:
 	"""
@@ -17,32 +17,28 @@ class dbmanager:
 	"""
 	
 	def __init__(self,
-		name,
 		dbpath,
 		dbname,
-		version,
 		date,
 		description,
 		readme = None,
 		program = None,
-		metaname = True,
+		scopes = ("Private", "Protected", "Public"),
 		username = None,
 		remote_dbhost = None,
 		remote_dbpath = None,
 		server = None,
-		server_username = False,
+		server_username = None,
 		port = None):
 		"""
 		Initialize a cluster CLI utility program
-		:param name: The name of the cluster/server
 		:param dbpath: The local database path
 		:param dbname: The database name
-		:param version: The version of the program
 		:param date: The date of the program's last revision
 		:param description: A description of the program
 		:param readme: The file path of a README.md file
 		:param program: The name of the program (defaults to name)
-		:param metaname: Subdirectory containing "manifest.toml"" (optional)
+		:param scopes: The names of the scopes in the database
 		:param username: Your username on the cluster
 		:param remote_dbhost: The remote database host
 		:param remote_dbpath: The remote database path
@@ -50,10 +46,8 @@ class dbmanager:
 		:param server_username: Your username on the gateway server (optional)
 		:param port: The local port for gateway server SSH forwarding
 		"""
-		self.name = name
 		self.dbpath = dbpath
 		self.dbname = dbname
-		self.version = version
 		if isinstance(date, datetime.date):
 			self.date = date
 		else:
@@ -61,10 +55,10 @@ class dbmanager:
 		self.description = description
 		self.readme = readme
 		if program is None:
-			self.program = name.casefold()
+			self.program = dbname.casefold()
 		else:
 			self.program = program
-		self.metaname = metaname
+		self.scopes = scopes
 		self.username = username
 		self.remote_dbhost = remote_dbhost
 		self.remote_dbpath = remote_dbpath
@@ -77,23 +71,41 @@ class dbmanager:
 		self._parser = None
 		self._args = None
 
+	def _add_db_args(self, parser):
+		"""
+		Add database parameters to a parser.
+		:param parser: The parser to update
+		"""
+		parser.add_argument("-n", "--db-name", action="store",
+			help=f"database name (default: {self.dbname})",
+			default=self.dbname)
+		parser.add_argument("-D", "--db-path", action="store",
+			help=f"database path (default: {self.dbpath})",
+			default=self.dbpath)
+
 	def _add_server_args(self, parser):
 		"""
 		Add server parameters to a parser.
 		:param parser: The parser to update
 		"""
-		parser.add_argument("-p", "--port", action="store",
-			help="port forwarding", default=self.port)
+		parser.add_argument("-R", "--remote-host", action="store",
+			help=f"remote database host (default: {self.remote_dbhost})",
+			default=self.remote_dbhost)
+		parser.add_argument("-P", "--remote-path", action="store",
+			help=f"remote database path (default: {self.remote_dbpath})",
+			default=self.remote_dbpath)
 		parser.add_argument("-u", "--user", action="store",
-			help=f"{self.name} user", default=self.username)
+			help="remote user (default: {self.username})",
+			default=self.username)
+		parser.add_argument("-p", "--port", action="store",
+			help="port forwarding",
+			default=self.port)
 		parser.add_argument("-L", "--login", action="store",
-			help="gateway server user", default=self.server_username)
+			help=f"gateway server user (default: {self.server_username})",
+			default=self.server_username)
 		parser.add_argument("-S", "--server", action="store",
-			help="gateway server host", default=self.server)
-		parser.add_argument("--remote-host", action="store",
-			help="remote database host", default=self.remote_dbhost)
-		parser.add_argument("--remote-path", action="store",
-			help="remote database path", default=self.remote_dbpath)
+			help=f"gateway server host (default: {self.server})",
+			default=self.server)
 	
 	def _add_subcommand_ls(self, subparsers):
 		"""
@@ -102,7 +114,8 @@ class dbmanager:
 		"""
 		cmd = subparsers.add_parser("ls", 
 			help="list all datasets")
-		cmd.add_argument("-l", "--long", action="store_true",
+		self._add_db_args(cmd)
+		cmd.add_argument("-l", "--details", action="store_true",
 			help="show extended details")
 		cmd.add_argument("-s", "--scope", action="store",
 			help="filter by scope")
@@ -116,7 +129,8 @@ class dbmanager:
 		"""
 		cmd = subparsers.add_parser("ls-cache", 
 			help="list cached datasets")
-		cmd.add_argument("-l", "--long", action="store_true",
+		self._add_db_args(cmd)
+		cmd.add_argument("-l", "--details", action="store_true",
 			help="show extended details")
 		cmd.add_argument("-s", "--scope", action="store",
 			help="filter by scope")
@@ -134,6 +148,7 @@ class dbmanager:
 		"""
 		cmd = subparsers.add_parser("search", 
 			help="search all datasets")
+		self._add_db_args(cmd)
 		cmd.add_argument("pattern", action="store",
 			help="search pattern (regex allowed)")
 		cmd.add_argument("-s", "--scope", action="store",
@@ -148,6 +163,7 @@ class dbmanager:
 		"""
 		cmd = subparsers.add_parser("search-cache", 
 			help="search cached datasets")
+		self._add_db_args(cmd)
 		cmd.add_argument("pattern", action="store",
 			help="search pattern (regex allowed)")
 		cmd.add_argument("-s", "--scope", action="store",
@@ -166,6 +182,7 @@ class dbmanager:
 			help="maximum cache size (10, 100, 1000, etc.)", type=float)
 		cmd.add_argument("units", action="store",
 			help="cache size units (MB, GB, TB, etc.)")
+		self._add_db_args(cmd)
 		cmd.add_argument("-s", "--scope", action="store",
 			help="filter by scope")
 		cmd.add_argument("-g", "--group", action="store",
@@ -192,6 +209,7 @@ class dbmanager:
 			help="describe a dataset")
 		cmd.add_argument("id", action="store",
 			help="the identifier of the dataset to describe")
+		self._add_db_args(cmd)
 	
 	def _add_subcommand_sync(self, subparsers):
 		"""
@@ -202,6 +220,7 @@ class dbmanager:
 			help="sync a dataset to local cache")
 		cmd.add_argument("id", action="store",
 			help="the identifier of the dataset to sync")
+		self._add_db_args(cmd)
 		self._add_server_args(cmd)
 		cmd.add_argument("-f", "--force", action="store_true",
 			help="force re-sync if already cached")
@@ -210,23 +229,6 @@ class dbmanager:
 		cmd.add_argument("--dry-run", action="store_true",
 			help="show what would happen without doing it?")
 	
-	def _add_subcommand_submit(self, subparsers):
-		"""
-		Add 'submit' subcommand to subparsers.
-		:param subparsers: The subparsers to update
-		"""
-		cmd = subparsers.add_parser("submit", 
-			help="submit a dataset")
-		cmd.add_argument("path", action="store",
-			help="the path to the dataset directory")
-		self._add_server_args(cmd)
-		cmd.add_argument("-f", "--force", action="store_true",
-			help="force re-submission if already tracked")
-		cmd.add_argument("--ask", action="store_true",
-			help="ask to confirm before submitting?")
-		cmd.add_argument("--dry-run", action="store_true",
-			help="show what would happen without doing it?")
-	
 	def _add_subcommand_status(self, subparsers):
 		"""
 		Add 'status' subcommand to subparsers.
@@ -234,7 +236,8 @@ class dbmanager:
 		"""
 		cmd = subparsers.add_parser("status", 
 			help="report status of cache against manifest")
-		cmd.add_argument("-l", "--long", action="store_true",
+		self._add_db_args(cmd)
+		cmd.add_argument("-l", "--details", action="store_true",
 			help="show extended details")
 		cmd.add_argument("-s", "--scope", action="store",
 			help="filter by scope")
@@ -269,7 +272,6 @@ class dbmanager:
 		self._add_subcommand_prune_cache(subparsers)
 		self._add_subcommand_describe(subparsers)
 		self._add_subcommand_sync(subparsers)
-		self._add_subcommand_submit(subparsers)
 		self._add_subcommand_status(subparsers)
 		if self.readme is not None:
 			self._add_subcommand_readme(subparsers)
@@ -293,7 +295,7 @@ class dbmanager:
 			raise NotADirectoryError(f"database does not exist: '{self.dbpath}'")
 		# connect and return database
 		db = expdb(self.username, self.dbpath, self.dbname,
-			metaname=self.metaname,
+			scopes=self.scopes,
 			remote_dbhost=self.remote_dbhost,
 			remote_dbpath=self.remote_dbpath,
 			server=self.server,
@@ -313,22 +315,33 @@ class dbmanager:
 		# version
 		if args.version:
 			description = self.description.splitlines()[0]
-			print(f"{description} version {self.version} (revised {self.date})")
+			print(f"{description} (revised {self.date})")
 			print(badwulf_attribution())
 			sys.exit()
 		# help
 		if args.cmd is None:
 			self._parser.print_help()
+		# readme
+		elif args.cmd == "readme":
+			if args.pager is None:
+				cmd = ["glow", "-p", "-w", str(args.width)]
+			else:
+				cmd = [args.pager]
+			cmd += [self.readme]
+			subprocess.run(cmd)
+		# ...
 		else:
+			self.dbname = args.db_name
+			self.dbpath = args.db_path
 			db = self.open_db()
 		# ls
 		if args.cmd == "ls":
 			datasets = db.ls(
 				scope=args.scope,
 				group=args.group,
-				details=args.long)
-			if args.long:
-				print_datasets(datasets)
+				details=args.details)
+			if args.details:
+				print(format_datasets(datasets))
 			else:
 				for name in datasets:
 					print(f"['{name}']")
@@ -338,7 +351,7 @@ class dbmanager:
 			datasets = db.ls_cache(
 				scope=args.scope,
 				group=args.group,
-				details=args.long or sort)
+				details=args.details or sort)
 			if sort:
 				if args.reverse is not None:
 					args.sort = args.reverse
@@ -351,13 +364,13 @@ class dbmanager:
 				elif sortby == "mtime".casefold():
 					datasets.sort(key=lambda x: x.mtime)
 				else:
-					sys.exit(f"msi ls-cache: error: can't sort by attribute: '{args.sort}'")
+					sys.exit(f"{self.program} ls-cache: error: can't sort by attribute: '{args.sort}'")
 				if args.reverse:
 					datasets.reverse()
-			if sort or args.long:
-				print_datasets(datasets)
+			if sort or args.details:
+				print(format_datasets(datasets))
 				sizes = [x.size for x in datasets]
-				print(f"~= {format_bytes(sum(sizes))} total")
+				print(f"~= {format_size(sum(sizes))} total")
 			else:
 				for name in datasets:
 					print(f"['{name}']")
@@ -367,14 +380,14 @@ class dbmanager:
 				pattern=args.pattern,
 				scope=args.scope,
 				group=args.group)
-			print_datasets(hits)
+			print(format_datasets(hits))
 		# search-cache
 		elif args.cmd == "search-cache":
 			hits = db.search_cache(
 				pattern=args.pattern,
 				scope=args.scope,
 				group=args.group)
-			print_datasets(hits)
+			print(format_datasets(hits))
 		# prune-cache
 		elif args.cmd == "prune-cache":
 			if args.strategy is None:
@@ -390,7 +403,7 @@ class dbmanager:
 		elif args.cmd == "describe":
 			dataset = db.get(args.id)
 			if dataset is None:
-				sys.exit(f"msi describe: error: no such dataset: '{args.id}'")
+				sys.exit(f"{self.program} describe: error: no such dataset: '{args.id}'")
 			else:
 				print(dataset.describe())
 				if args.id in db.cache:
@@ -398,46 +411,32 @@ class dbmanager:
 		# sync
 		elif args.cmd == "sync":
 			if args.id in db.cache and not args.force:
-				sys.exit("msi sync: dataset is already cached; use --force to re-sync")
+				sys.exit(f"{self.program} sync: dataset is already cached; use --force to re-sync")
 			db.username = args.user
+			db.port = args.port
 			db.remote_dbhost = args.remote_host
 			db.remote_dbpath = args.remote_path
-			db.server = args.server
 			db.server_username = args.login
-			db.port = args.port
+			db.server = args.server
 			db.sync(args.id,
 				force=args.force,
 				dry_run=args.dry_run, ask=args.ask)
-		# submit
-		elif args.cmd == "submit":
-			name = os.path.basename(args.path)
-			if name in db.manifest and not args.force:
-				sys.exit("msi submit: dataset is already tracked; use --force to re-submit")
-			db.username = args.user
-			db.remote_dbhost = args.remote_host
-			db.remote_dbpath = args.remote_path
-			db.server = args.server
-			db.server_username = args.login
-			db.port = args.port
-			db.submit(args.path,
-				force=args.force,
-				dry_run=args.dry_run, ask=args.ask)
 		# status
 		elif args.cmd == "status":
 			synced, remoteonly, localonly = db.status(
 				scope=args.scope,
 				group=args.group,
-				details=args.long)
-			msg_synced = "\n~~~~ synced:"
-			msg_remoteonly = "\n>>>> tracked but not cached:"
-			msg_localonly = "\n<<<< cached but not tracked:"
-			if args.long:
+				details=args.details)
+			msg_synced = f"\n~~~~ {len(synced)} synced ~~~~"
+			msg_remoteonly = f"\n>>>> {len(remoteonly)} tracked but not cached >>>>"
+			msg_localonly = f"\n<<<< {len(localonly)} cached but not tracked <<<<"
+			if args.details:
 				print(msg_synced)
-				print_datasets(synced)
+				print(format_datasets(synced))
 				print(msg_remoteonly)
-				print_datasets(remoteonly)
+				print(format_datasets(remoteonly))
 				print(msg_localonly)
-				print_datasets(localonly)
+				print(format_datasets(localonly))
 			else:
 				print(msg_synced)
 				for name in synced:
@@ -448,12 +447,4 @@ class dbmanager:
 				print(msg_localonly)
 				for name in localonly:
 					print(f"['{name}']")
-		# readme
-		elif args.cmd == "readme":
-			if args.pager is None:
-				cmd = ["glow", "-p", "-w", str(args.width)]
-			else:
-				cmd = [args.pager]
-			cmd += [self.readme]
-			subprocess.run(cmd)
 		sys.exit()
