@@ -9,7 +9,6 @@ else:
 	import tomli as tomllib
 
 import shutil
-from time import sleep
 from dataclasses import dataclass
 from dataclasses import asdict
 from datetime import datetime
@@ -20,18 +19,70 @@ from .tools import fix_path
 from .tools import dirsize
 from .tools import dirfiles
 from .tools import to_bytes
-from .tools import format_bytes
-from .tools import format_datasets
-from .tools import print_datasets
 from .tools import askYesNo
 from .tools import grep1
 from .tools import grepl
 
-# scopes
-SCOPE_PRIVATE = "Private"
-SCOPE_PROTECTED = "Protected"
-SCOPE_PUBLIC = "Public"
-SCOPE_XFER = "Xfer"
+def format_size(x, units = "auto"):
+	"""
+	Format bytes
+	:param x: The number of bytes
+	:param units: The units (B, KB, MB, etc.)
+	:returns: A string
+	"""
+	if units == "auto":
+		if x >= 1000 ** 5:
+			units = "PB"
+		elif x >= 1000 ** 4:
+			units = "TB"
+		elif x >= 1000 ** 3:
+			units = "GB"
+		elif x >= 1000 ** 2:
+			units = "MB"
+		elif x >= 1000:
+			units = "KB"
+		else:
+			units = "bytes"
+	if units in ("bytes", "B"):
+		if x == 1 and units == "bytes":
+			units = "byte"
+		x = int(x)
+	else:
+		if units == "KB":
+			x /= 1000
+		elif units == "MB":
+			x /= 1000 ** 2
+		elif units == "GB":
+			x /= 1000 ** 3
+		elif units == "TB":
+			x /= 1000 ** 4
+		elif units == "PB":
+			x /= 1000 ** 5
+		else:
+			raise ValueError(f"invalid units: {units}")
+		x = round(x, ndigits=2)
+		x = float(x)
+	return f"{x} {units}"
+
+def format_datasets(iterable, names_only = False, header = True):
+	"""
+	Format datasets
+	:param iterable: An iterable of datasets
+	:param names_only: Print names only?
+	:param header: Print number of datasets?
+	:return: A formatted string
+	"""
+	if names_only:
+		sl = [f"['{dataset.name}']{dataset.flag}" 
+			for dataset 
+			in iterable]
+	else:
+		sl = [f"['{dataset.name}']{dataset.flag}\n{dataset}" 
+			for dataset 
+			in iterable]
+	if header:
+		sl = [f"#### {len(sl)} datasets ####\n"] + sl
+	return "\n".join(sl)
 
 @dataclass
 class expdata:
@@ -146,6 +197,108 @@ class expdata:
 		:returns: bool
 		"""
 		return grep1(pattern, self.group) is not None
+	
+	@property
+	def flag(self):
+		"""
+		String to display to flag the dataset
+		"""
+		return ""
+
+@dataclass
+class expcache:
+	"""
+	File metadata for a cached dataset
+	"""
+
+	name: str
+	path: str
+	_atime: float
+	_mtime: float
+	size: int
+	err: list
+	
+	def __init__(self, name, path, atime, mtime, size, err = None):
+		"""
+		Initialize an expcache instance
+		:param name: The name of the dataset
+		:param path: The file path to the dataset
+		:param atime: Last access time
+		:param mtime: Last modified time
+		:param size: Total size in storage
+		:param err: Problems with the cached dataset
+		"""
+		self.name = name
+		self.path = path
+		self._atime = atime
+		self._mtime = mtime
+		self.size = size
+		if err is None:
+			self.err = []
+		else:
+			self.err = err
+	
+	@property
+	def atime(self):
+		"""
+		Get last accessed time
+		"""
+		return datetime.fromtimestamp(self._atime)
+	
+	@property
+	def mtime(self):
+		"""
+		Get last modified time
+		"""
+		return datetime.fromtimestamp(self._mtime)
+	
+	def __str__(self):
+		"""
+		Return str(self)
+		"""
+		return self.describe()
+	
+	def describe(self, printformats = None):
+		"""
+		Return cached dataset description
+		"""
+		path = f" path: '{self.path}'"
+		atime = f" atime: '{self.atime}'"
+		mtime = f" mtime: '{self.mtime}'"
+		size = f" size: {format_size(self.size)}"
+		sl = [path, atime, mtime, size]
+		if len(self.err) > 0:
+			errors_header = f" errors: {len(self.err)}"
+			errors = []
+			for issue in self.err.keys():
+				issue_list = [f"  {issue} {key}: {val}" 
+					for key, val
+					in self.err[issue].items()]
+				errors.extend(issue_list)
+			errors = [errors_header] + [" {"] + errors + [" }"]
+			sl.extend(errors)
+		if printformats is not None:
+			files = []
+			fmts = [f"{fmt}$" for fmt in printformats]
+			for pattern in fmts:
+				files.extend(dirfiles(self.path, pattern, recursive=True))
+			files = [f" file {str(i + 1)}: '{file}'" 
+				for i, file 
+				in enumerate(files)]
+			sl.extend(files)
+			sl = [" " + s for s in sl]
+			sl = [" Cached:", " {"] + sl + [" }"]
+		return "\n".join(["{"] + sl + ["}"])
+	
+	@property
+	def flag(self):
+		"""
+		String to display to flag the dataset
+		"""
+		if len(self.err) > 0:
+			return " !!!"
+		else:
+			return ""
 
 @dataclass
 class expsearch:
@@ -224,82 +377,24 @@ class expsearch:
 			sl.append(f"  >{field}: {match}")
 		sl.append(" }")
 		return "\n".join(["{"] + sl + ["}"])
-
-@dataclass
-class expcache:
-	"""
-	File metadata for a cached dataset
-	"""
-
-	name: str
-	path: str
-	_atime: float
-	_mtime: float
-	size: int
-	
-	def __init__(self, name, path, atime, mtime, size):
-		"""
-		Initialize an expcache instance
-		:param name: The name of the dataset
-		:param path: The file path to the dataset
-		:param atime: Last access time
-		:param mtime: Last modified time
-		:param size: Total size in storage
-		"""
-		self.name = name
-		self.path = path
-		self._atime = atime
-		self._mtime = mtime
-		self.size = size
 	
 	@property
-	def atime(self):
+	def flag(self):
 		"""
-		Get last accessed time
+		String to display to flag the dataset
 		"""
-		return datetime.fromtimestamp(self._atime)
-	
-	@property
-	def mtime(self):
-		"""
-		Get last modified time
-		"""
-		return datetime.fromtimestamp(self._mtime)
-	
-	def __str__(self):
-		"""
-		Return str(self)
-		"""
-		return self.describe()
-	
-	def describe(self, printformats = None):
-		"""
-		Return cached dataset description
-		"""
-		path = f" path: '{self.path}'"
-		atime = f" atime: '{self.atime}'"
-		mtime = f" mtime: '{self.mtime}'"
-		size = f" size: {format_bytes(self.size)}"
-		sl = [path, atime, mtime, size]
-		if printformats is not None:
-			files = []
-			fmts = [f"{fmt}$" for fmt in printformats]
-			for pattern in fmts:
-				files.extend(dirfiles(self.path, pattern, recursive=True))
-			files = [f" file {str(i + 1)}: '{file}'" 
-				for i, file 
-				in enumerate(files)]
-			sl.extend(files)
-			sl = [" " + s for s in sl]
-			sl = [" Cached:", " {"] + sl + [" }"]
-		return "\n".join(["{"] + sl + ["}"])
+		if len(self.hits) > 0:
+			return f" >> {len(self.hits)}"
+		else:
+			return ""
 
 class expdb:
 	"""
 	Database manager for experimental datasets and metadata
 	"""
 	
-	def __init__(self, username, dbpath, dbname, metaname = True,
+	def __init__(self, username, dbpath, dbname,
+		scopes = ("Private", "Protected", "Public"),
 		remote_dbhost = None, remote_dbpath = None,
 		server = None, server_username = None,
 		port = 8080, remote_port = 22, verbose = False,
@@ -307,9 +402,9 @@ class expdb:
 		"""
 		Initialize an expdb instance
 		:param username: Your username on remote database host
-		:param dbpath: The local database path
+		:param dbpath: The local database path (must contain "manifest.toml")
 		:param dbname: The database name (may be None)
-		:param metaname: Subdirectory containing "manifest.toml"" (optional)
+		:param scopes: The names of the scopes in the database
 		:param remote_dbhost: The remote database host
 		:param remote_dbpath: The remote database path
 		:param server: The gateway server hostname (optional)
@@ -324,12 +419,7 @@ class expdb:
 		self.username = username
 		self.dbpath = fix_path(dbpath, must_exist=True)
 		self.dbname = dbname
-		if metaname in (True, False):
-			if metaname:
-				metaname = dbname
-			else:
-				metaname = None
-		self.metaname = metaname
+		self.scopes = scopes
 		self.remote_dbhost = remote_dbhost
 		self.remote_dbpath = remote_dbpath
 		self.server = server
@@ -467,10 +557,7 @@ class expdb:
 		"""
 		Refresh the database manifest
 		"""
-		if self.metaname is None:
-			path = os.path.join(self.dbdir, "manifest.toml")
-		else:
-			path = os.path.join(self.dbdir, self.metaname, "manifest.toml")
+		path = os.path.join(self.dbdir, "manifest.toml")
 		path = fix_path(path, must_exist=True)
 		if self.verbose:
 			print(f"parsing '{path}'")
@@ -489,9 +576,8 @@ class expdb:
 		if self.verbose:
 			print("detecting cached datasets")
 		cache = []
-		cache.extend(self._get_cached_scope(SCOPE_PRIVATE))
-		cache.extend(self._get_cached_scope(SCOPE_PROTECTED))
-		cache.extend(self._get_cached_scope(SCOPE_PUBLIC))
+		for scope in self.scopes:
+			cache.extend(self._get_cached_scope(scope))
 		self._cache = {dataset.name: dataset for dataset in cache}
 		if self.verbose:
 			print(f"{len(cache)} datasets available locally")
@@ -509,8 +595,21 @@ class expdb:
 		size = dirsize(path, all_names=True)
 		atime = os.path.getatime(path)
 		mtime = os.path.getmtime(path)
+		expected = self.get(dataset)
+		if expected is None:
+			err = None
+		else:
+			err = {}
+			if scope.casefold() != expected.scope.casefold():
+				err["scope"] = {
+					"expected": expected.scope,
+					"detected": scope}
+			if group.casefold() != expected.group.casefold():
+				err["group"] = {
+					"expected": expected.group,
+					"detected": group}
 		return expcache(name=dataset, path=path,
-			atime=atime, mtime=mtime, size=size)
+			atime=atime, mtime=mtime, size=size, err=err)
 	
 	def _get_cached_group(self, scope, group):
 		"""
@@ -677,11 +776,11 @@ class expdb:
 		while delsize[target] > limit:
 			target += 1
 		newsize = totsize - sum(sizes[:target])
-		print(f"the local cache is currently {format_bytes(totsize)}")
+		print(f"the local cache is currently {format_size(totsize)}")
 		print(f"using strategy: {strategy}")
 		print("the following datasets will be deleted from the cache:")
-		print_datasets(cache[:target], names_only=True)
-		print(f"~= {format_bytes(sum(sizes[:target]))} will be freed")
+		print(format_datasets(cache[:target], names_only=True))
+		print(f"~= {format_size(sum(sizes[:target]))} will be freed")
 		if not target:
 			return
 		if ask and not askYesNo():
@@ -693,7 +792,7 @@ class expdb:
 					shutil.rmtree(fix_path(x.path, must_exist=True))
 				except Exception:
 					print(f"failed to delete '{x.path}'")
-			print(f"the local cache is now {format_bytes(newsize)}")
+			print(f"the local cache is now {format_size(newsize)}")
 		return
 	
 	def sync(self, name, force = False, dry_run = False, ask = False):
@@ -731,7 +830,6 @@ class expdb:
 				server_username=self.server_username,
 				port=self.port,
 				destination_port=self.remote_port)
-			sleep(1) # allow time to connect
 			con.download(src, dest, dry_run=dry_run, ask=ask)
 			if os.path.isdir(dest):
 				print("sync complete; refreshing cache metadata")
@@ -741,46 +839,6 @@ class expdb:
 		finally:
 			con.close()
 	
-	def submit(self, path, force = False, dry_run = False, ask = False):
-		"""
-		Submit a dataset to the database maintainer(s)
-		:param path: The path to the data directory
-		:param force: Should the dataset be re-submitted if already tracked?
-		:param dry_run: Show what would be done without doing it?
-		:param ask: Confirm before uploading?
-		"""
-		path = fix_path(path, must_exist=False)
-		name = os.path.basename(path)
-		if name in self.manifest and not force:
-			print("dataset is already tracked; use force=True to re-submit")
-			return
-		if self.remote_dbhost is None:
-			raise IOError("remote host is None")
-		if self.remote_dbpath is None:
-			raise IOError("remote path is None")
-		if not os.path.isdir(path):
-			raise NotADirectoryError(f"path must be a directory: {path}")
-		if path[-1] != "/":
-			src = path + "/"
-		else:
-			src = path
-		dest = os.path.join(self.remote_dbdir, SCOPE_XFER, name)
-		dest = dest + "/"
-		try:
-			con = rssh(self.username,
-				destination=self.remote_dbhost,
-				server=self.server,
-				server_username=self.server_username,
-				port=self.port,
-				destination_port=self.remote_port)
-			sleep(1) # allow time to connect
-			con.upload(src, dest, dry_run=dry_run, ask=ask)
-			print("submission complete")
-		except Exception:
-			print("a problem occured during submission")
-		finally:
-			con.close()
-	
 	def status(self, scope = None, group = None, details = False):
 		"""
 		Get status of cache versus manifest
