@@ -14,13 +14,11 @@ def badwulf_version():
 	"""
 	return version("badwulf")
 
-def badwulf_attribution():
+def badwulf_attribution(copyright = True):
 	"""
 	Get badwulf package attribution
 	"""
-	str1 = "powered by badwulf |"
-	str2 = "copyright (c) 2024 Kylie A. Bemis"
-	return str1 + " " + str2
+	return "powered by badwulf v" + badwulf_version()
 
 def is_known_host(nodes):
 	"""
@@ -55,84 +53,6 @@ def to_bytes(x, units = "bytes"):
 		raise ValueError(f"invalid units: {units}")
 	return x
 
-def format_bytes(x, units = "auto"):
-	"""
-	Format bytes
-	:param x: The number of bytes
-	:param units: The units (B, KB, MB, etc.)
-	:returns: A string
-	"""
-	if units == "auto":
-		if x >= 1000 ** 5:
-			units = "PB"
-		elif x >= 1000 ** 4:
-			units = "TB"
-		elif x >= 1000 ** 3:
-			units = "GB"
-		elif x >= 1000 ** 2:
-			units = "MB"
-		elif x >= 1000:
-			units = "KB"
-		else:
-			units = "bytes"
-	if units in ("bytes", "B"):
-		if x == 1 and units == "bytes":
-			units = "byte"
-		x = int(x)
-	else:
-		if units == "KB":
-			x /= 1000
-		elif units == "MB":
-			x /= 1000 ** 2
-		elif units == "GB":
-			x /= 1000 ** 3
-		elif units == "TB":
-			x /= 1000 ** 4
-		elif units == "PB":
-			x /= 1000 ** 5
-		else:
-			raise ValueError(f"invalid units: {units}")
-		x = round(x, ndigits=2)
-		x = float(x)
-	return f"{x} {units}"
-
-def print_bytes(x, units = "auto"):
-	"""
-	Print bytes
-	:param x: The number of bytes
-	:param units: The units (B, KB, MB, etc.)
-	"""
-	print(format_bytes(x, units))
-
-def format_datasets(iterable, names_only = False, header = True):
-	"""
-	Format datasets
-	:param iterable: An iterable of datasets
-	:param names_only: Print names only?
-	:param header: Print number of datasets?
-	:return: A formatted string
-	"""
-	if names_only:
-		sl = [f"['{dataset.name}']" 
-			for dataset 
-			in iterable]
-	else:
-		sl = [f"['{dataset.name}']\n{dataset}" 
-			for dataset 
-			in iterable]
-	if header:
-		sl = [f"#### {len(sl)} datasets ####\n"] + sl
-	return "\n".join(sl)
-
-def print_datasets(iterable, names_only = False, header = True):
-	"""
-	Print datasets
-	:param iterable: An iterable of datasets
-	:param names_only: Print names only?
-	:param header: Print number of datasets?
-	"""
-	print(format_datasets(iterable, names_only, header))
-
 def askYesNo(msg = "Continue? (yes/no): "):
 	"""
 	Ask a user to confirm yes or no
@@ -148,6 +68,16 @@ def askYesNo(msg = "Continue? (yes/no): "):
 		else:
 			print("Invalid input. Please enter yes/no.")
 
+def getenv(key, default = None, empty = ""):
+	"""
+	Get an shell environment variable
+	:param key: The environment variable name
+	:param default: The value to use if unset
+	:param empty: The value to use for empty strings
+	"""
+	value = os.getenv(key, default=default)
+	return value if value != "" else empty
+
 def squote(s, q = "'"):
 	"""
 	Wrap a string in quotes
@@ -170,7 +100,7 @@ def dquote(s, q = '"'):
 	else:
 		return s
 
-def fix_path(path, must_exist = True, escape_spaces = True):
+def fix_path(path, must_exist = True, escape_spaces = False):
 	"""
 	Normalize and expand paths
 	:param path: The path to normalize
@@ -180,10 +110,10 @@ def fix_path(path, must_exist = True, escape_spaces = True):
 	if "~" in path:
 		path = os.path.expanduser(path)
 	path = os.path.realpath(path)
-	if escape_spaces:
-		path = path.replace(" ", r"\ ")
 	if must_exist and not os.path.exists(path):
 		raise FileNotFoundError(f"path does not exist: '{path}'")
+	if escape_spaces:
+		path = path.replace(" ", r"\ ")
 	return path
 
 def file_create(path):
